export type GlobalDirectory = {
	/**
	The directory with globally installed packages.

	Equivalent to `npm root --global`.
	*/
	readonly packages: string;

	/**
	The directory with globally installed binaries.

	Equivalent to `npm bin --global`.
	*/
	readonly binaries: string;

	/**
	The directory with directories for packages and binaries. You probably want either of the above.

	Equivalent to `npm prefix --global`.
	*/
	readonly prefix: string;
};

declare const globalDirectory: {
	/**
	Get the directory of globally installed packages and binaries.

	@example
	```
	import globalDirectory from 'global-directory';

	console.log(globalDirectory.npm.prefix);
	//=> '@@HOMEBREW_PREFIX@@'

	console.log(globalDirectory.npm.packages);
	//=> '@@HOMEBREW_PREFIX@@/lib/node_modules'
	```
	*/
	readonly npm: GlobalDirectory;

	/**
	Get the directory of globally installed packages and binaries.

	@example
	```
	import globalDirectory from 'global-directory';

	console.log(globalDirectory.npm.binaries);
	//=> '@@HOMEBREW_PREFIX@@/bin'

	console.log(globalDirectory.yarn.packages);
	//=> '/Users/sindresorhus/.config/yarn/global/node_modules'
	```
	*/
	readonly yarn: GlobalDirectory;
};

export default globalDirectory;
